from niprov.config import Configuration


class Dependencies(object):

    def __init__(self, config=None):
        if config is None:
            config = Configuration()
        self.config = config

    def reconfigureOrGetConfiguration(self, newConfiguration):
        if newConfiguration is not None:
            self.config = newConfiguration
        return self.config

    def getConfiguration(self):
        return self.config

    def getExternals(self):
        import niprov.externals
        return niprov.externals.Externals()

    def getExportFactory(self):
        import niprov.exporters
        return niprov.exporters.ExportFactory(dependencies=self)

    def getFileFactory(self):
        import niprov.files
        return niprov.files.FileFactory(dependencies=self)

    def getFileFilter(self):
        import niprov.filefilter
        return niprov.filefilter.FileFilter(dependencies=self)

    def getFilesystem(self):
        import niprov.filesystem
        return niprov.filesystem.Filesystem()

    def getHasher(self):
        import niprov.hashing
        return niprov.hashing.Hasher()

    def getLibraries(self):
        import niprov.libraries
        return niprov.libraries.Libraries()

    def getListener(self):
        import niprov.commandline
        return niprov.commandline.Commandline(self.config)

    def getLocationFactory(self):
        import niprov.locationfactory
        return niprov.locationfactory.LocationFactory(dependencies=self)

    def getNarrator(self):
        import niprov.narrator
        return niprov.narrator.Narrator()

    def getRepository(self):
        import niprov.jsonfile
        import niprov.mongo
        if self.config.database_type == 'file':
            return niprov.jsonfile.JsonFile(dependencies=self)
        elif self.config.database_type == 'MongoDB':
            return niprov.mongo.MongoRepository(dependencies=self)

    def getSerializer(self):
        import niprov.jsonserializing
        return niprov.jsonserializing.JsonSerializer()




#!/usr/bin/python
# -*- coding: UTF-8 -*-
import os
from niprov.dependencies import Dependencies
from niprov.adding import add


def discover(root, dependencies=Dependencies()):
    """
    Search a directory for image files, and add them to your provenance collection.

    Files are only included if they match the filters in the 
    'discover_file_extensions' settings.
    Refer to niprov.add for details on what happens to individual files.

    Args:
        root (str): The top directory in which to look for new files.
    """
    filesys = dependencies.getFilesystem()
    filefilter = dependencies.getFileFilter()
    listener = dependencies.getListener()

    dirs = filesys.walk(root)
    stats = {'total':0, 'new':0, 'series':0, 'failed':0, 'known':0}
    for (root, sdirs, files) in dirs:
        for filename in files:
            filepath = os.path.join(root, filename)
            if filefilter.include(filename):
                stats['total'] = stats['total'] + 1
                (p, status) = add(filepath, transient=False, 
                    dependencies=dependencies)
                stats[status] = stats[status] + 1
    listener.discoveryFinished(nnew=stats['new'], nadded=stats['series'], 
        nfailed=stats['failed'], ntotal=stats['total'])




from niprov.dependencies import Dependencies
from niprov.location import Location


class LocationFactory(object):
    """Creates Location objects."""

    def __init__(self, dependencies=Dependencies()):
        self.dependencies = dependencies

    def fromString(self, locationString):
        """Creates a Location object.
        
        Args:
            locationString (str): String with path and optionally computer id. 
        """
        return Location(locationString)

    def completeString(self, locationString):
        """Validates a location string.
        
        If locationString only contains a filesystem path, computer info will
        be added.
        """
        return str(self.fromString(locationString))



import socket


class Location(object):
    """Represents the location of a file."""

    def __init__(self, locationString):
        if ':' in locationString:
            parts = locationString.split(':')
            self.hostname = parts[0]
            self.path = parts[1]
        else:
            self.hostname = socket.gethostname()
            self.path = locationString

    def toDictionary(self):
        d = {}
        d['path'] = self.path
        d['hostname'] = self.hostname
        d['location'] = str(self)
        return d

    def __str__(self):
        return self.toString()

    def toString(self):
        return ':'.join([self.hostname, self.path])



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.dependencies import Dependencies


def inspect(location, dependencies=Dependencies()):
    fileFactory = dependencies.getFileFactory()
    return fileFactory.locatedAt(location).inspect()





#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.dependencies import Dependencies


def report(medium=None, form=None, forFile=None, forSubject=None, 
        statistics=False, dependencies=Dependencies()):
    """Publish or simply return provenance for selected files.

    To get provenance on one specific file, pass its path as the 'forFile' 
    argument. Alternatively, to get all files associated with a certain subject,
    use the 'forSubject' argument. If none of these is used, provenance for all
    files is reported.

    Args:
        medium (str): The medium in which to publish the provenance. 
            One of 'stdout' (print the provenance to the terminal) or None.
        forFile (str): Select one file based on this path.
        forSubject (str): Select files regarding this subject.
        statistics (bool): Print overall statistics.

    Returns:
        Provenance reported. Either a list of dicts, or a dict.
    """
    exportFactory = dependencies.getExportFactory()
    repository = dependencies.getRepository()
    listener = dependencies.getListener()
    location = dependencies.getLocationFactory()

    exporter = exportFactory.createExporter(medium, form)
    if statistics:
        provenance = repository.statistics()
    elif forFile:
        forFile = location.completeString(forFile)
        if not repository.knowsByLocation(forFile):
            listener.unknownFile(forFile)
            return
        provenance = repository.byLocation(forFile)
    elif forSubject:
        provenance = repository.bySubject(forSubject)
    else:
        provenance = repository.all()
    return exporter.export(provenance)




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.dependencies import Dependencies


class FileFilter(object):

    def __init__(self, dependencies=Dependencies()):
        config = dependencies.getConfiguration()
        self.filters = config.discover_file_extensions

    def include(self, filepath):
        """Whether the file is to be included in discovery.

        Args:
            filepath (str): The full path of the file.

        Returns:
            bool: True if the file should be included.
        """
        for filt in self.filters:
            if filt in filepath:
                return True
        return False



import unittest
import mock
from mock import Mock, patch


class DependenciesTests(unittest.TestCase):

    def setUp(self):
        pass

    def test_Changing_setting_directly_affects_component_setting(self):
        from niprov.dependencies import Dependencies
        dependencies = Dependencies()
        self.assertFalse(dependencies.getListener().settings.verbose)
        dependencies.config.verbose = True
        self.assertTrue(dependencies.getListener().settings.verbose)

    def test_Changing_storage_setting_changes_repository_provided(self):
        from niprov.dependencies import Dependencies
        from niprov.jsonfile import JsonFile
        from niprov.mongo import MongoRepository
        dependencies = Dependencies()
        with patch('niprov.mongo.pymongo'):
            dependencies.config.database_type = 'file'
            self.assertIsInstance(dependencies.getRepository(), JsonFile)
            dependencies.config.database_type = 'MongoDB'
            self.assertIsInstance(dependencies.getRepository(), MongoRepository)

    def test_reconfigureOrGetConfiguration_with_None_doesnt_affect_config(self):
        from niprov.dependencies import Dependencies
        dependencies = Dependencies()
        self.assertFalse(dependencies.getListener().settings.verbose)
        dependencies.reconfigureOrGetConfiguration(None)
        self.assertFalse(dependencies.getListener().settings.verbose)

    def test_reconfigureOrGetConfiguration_with_None_returns_config(self):
        from niprov.dependencies import Dependencies
        dependencies = Dependencies()
        outConfig = dependencies.reconfigureOrGetConfiguration(None)
        self.assertEqual(dependencies.config, outConfig)

    def test_reconfigureOrGetConfiguration_with_new_config_returns_it(self):
        from niprov.dependencies import Dependencies, Configuration
        dependencies = Dependencies()
        newSettings = Configuration()
        newSettings.verbose = True
        outConfig = dependencies.reconfigureOrGetConfiguration(newSettings)
        self.assertEqual(newSettings, outConfig)

    def test_reconfigureOrGetConfiguration_creates_new_dependencies(self):
        from niprov.dependencies import Dependencies, Configuration
        dependencies = Dependencies()
        self.assertFalse(dependencies.getListener().settings.verbose)
        newSettings = Configuration()
        newSettings.verbose = True
        dependencies.reconfigureOrGetConfiguration(newSettings)
        self.assertTrue(dependencies.getListener().settings.verbose)

    def test_GetConfiguration_returns_config(self):
        from niprov.dependencies import Dependencies
        dependencies = Dependencies()
        outConfig = dependencies.getConfiguration()
        self.assertEqual(dependencies.config, outConfig)

    def test_Filefactory_provided(self):
        from niprov.dependencies import Dependencies
        from niprov.files import FileFactory
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getFileFactory(), FileFactory)

    def test_provides_LocationFactory(self):
        from niprov.dependencies import Dependencies
        from niprov.locationfactory import LocationFactory
        dependencies = Dependencies()
        self.assertIsInstance(dependencies.getLocationFactory(), 
            LocationFactory)




import unittest
from mock import Mock
import os


class FileFilterTests(unittest.TestCase):

    def test_reads_extensions_from_config(self):
        import niprov.filefilter
        dependencies = Mock()
        dependencies.getConfiguration().discover_file_extensions = ['.yes',
            '.ofc']
        filt = niprov.filefilter.FileFilter(dependencies=dependencies)
        self.assertTrue(filt.include('/p/sth.ofc'))
        self.assertTrue(filt.include('/p/sth.yes'))
        self.assertFalse(filt.include('/p/sth.nop'))





from pyramid.config import Configurator
import waitress
from niprov.dependencies import Dependencies


def serve():
    wsgiapp = main(None)
    waitress.serve(wsgiapp, host='0.0.0.0', port=6543)

def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """
#    settings = {}
#    settings['reload_all'] = True
#    settings['debug_all'] = True
#    settings['mako.directories'] = os.path.join(here, 'templates')
    config = Configurator(settings=settings)
    config.include('pyramid_mako')
    config.add_static_view('static', 'static', cache_max_age=10)
    config.add_route('home', '/')
    config.add_route('latest', '/latest')
    config.add_route('stats', '/stats')
    config.add_route('subject', '/subject/{subject}')
    config.add_route('short', '/id/{id}')
    config.add_route('location', '/location/{host}*path')
    config.add_request_method(lambda r: Dependencies(), 
        'dependencies', reify=True)
    config.scan()
    return config.make_wsgi_app()

"""
Documentation:
PasteDeploy: http://pythonpaste.org/deploy/
(Handles reading config from file and picking server and app settings)
Waitress: http://waitress.readthedocs.org/en/latest/
(How to start and configure waitress)
"""



from pyramid.view import view_config
import os


@view_config(route_name='home', renderer='templates/home.mako')
def home(request):
    return {'project': 'niprov'}

@view_config(route_name='latest', renderer='templates/list.mako')
def latest(request):
    repository = request.dependencies.getRepository()
    return {'images':repository.latest()}

@view_config(route_name='subject', renderer='templates/list.mako')
def subject(request):
    subj = request.matchdict['subject']
    repository = request.dependencies.getRepository()
    return {'images':repository.bySubject(subj)}

@view_config(route_name='short', renderer='templates/single.mako')
def short(request):
    sid = request.matchdict['id']
    repository = request.dependencies.getRepository()
    return {'image':repository.byId(sid)}

@view_config(route_name='location', renderer='templates/single.mako')
def location(request):
    path = os.sep + os.path.join(*request.matchdict['path'])
    loc = request.matchdict['host'] + ':' + path
    repository = request.dependencies.getRepository()
    return {'image':repository.byLocation(loc)}

@view_config(route_name='stats', renderer='templates/stats.mako')
def stats(request):
    repository = request.dependencies.getRepository()
    return {'stats':repository.statistics()}




import unittest
from mock import Mock

class ExportFactoryTests(unittest.TestCase):

    def setUp(self):
        self.dependencies = Mock()
        from niprov.exporters import ExportFactory
        self.factory = ExportFactory(dependencies=self.dependencies)

    def test_Default(self):
        from niprov.directexporter import DirectExporter
        exporter = self.factory.createExporter(None, None)
        self.assertIsInstance(exporter, DirectExporter)

    def test_StdOut(self):
        from niprov.stdout import StandardOutputExporter
        exporter = self.factory.createExporter('stdout', None)
        self.assertIsInstance(exporter, StandardOutputExporter)

    def test_Form_supplied(self):
        exporter = self.factory.createExporter(None, 'narrative')
        self.assertEqual(exporter.form, 'narrative')
        exporter = self.factory.createExporter('stdout', 'narrative')
        self.assertEqual(exporter.form, 'narrative')







#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.dependencies import Dependencies


def markForApproval(files, dependencies=Dependencies()):
    """Mark a list of files for approval by a human.

    Args:
        files (list): List of paths of files tracked by niprov to mark for
            approval.
    """
    repository = dependencies.getRepository()
    for filepath in files:
        repository.updateApproval(filepath,'pending')

def markedForApproval(dependencies=Dependencies()):
    """List files marked for approval by a human.
    """
    repository = dependencies.getRepository() 
    listener = dependencies.getListener() 
    markedFiles = repository.byApproval('pending')
    listener.filesMarkedForApproval(markedFiles)
    return markedFiles

def approve(filepath, dependencies=Dependencies()):
    """Mark this file as approved.

    Args:
        filepath (str): Path to the tracked file that has been found valid.
    """
    repository = dependencies.getRepository() 
    repository.updateApproval(filepath,'granted')

def selectApproved(files, dependencies=Dependencies()):
    """Return only files that have approval status 'granted'.

    Args:
        files (list): List of paths of files to check for approval status.
    """
    repository = dependencies.getRepository() 
    location = dependencies.getLocationFactory()
    selection = []
    for filepath in files:
        locationString = location.completeString(filepath)
        img = repository.byLocation(locationString)
        if 'approval' in img.provenance:
            if img.provenance['approval'] == 'granted':
                selection.append(img.path)
    return selection



import pymongo
from niprov.dependencies import Dependencies


class MongoRepository(object):

    def __init__(self, dependencies=Dependencies()):
        self.config = dependencies.getConfiguration()
        self.factory = dependencies.getFileFactory()
        client = pymongo.MongoClient(self.config.database_url)
        self.db = client.get_default_database()

    def byLocation(self, locationString):
        """Get the provenance for a file at the given location. 

        In the case of a dicom series, this returns the provenance for the 
        series.

        Args:
            locationString (str): Location of the image file.

        Returns:
            dict: Provenance for one image file.
        """
        record = self.db.provenance.find_one({'location':locationString})
        return self.factory.fromProvenance(record)

    def knowsByLocation(self, locationString):
        """Whether the file at this location has provenance associated with it.

        Returns:
            bool: True if provenance is available for that path.
        """
        return self.db.provenance.find_one(
            {'location':locationString}) is not None

    def knows(self, image):
        """Whether this file has provenance associated with it.

        Returns:
            bool: True if provenance is available for this image.
        """
        return self.knowsByLocation(image.location.toString())

    def getSeries(self, image):
        """Get the object that carries provenance for the series that the image 
        passed is in. 

        Args:
            image (:class:`.DicomFile`): File that is part of a series.

        Returns:
            :class:`.DicomFile`: Image object that caries provenance for the series.
        """
        seriesUid = image.getSeriesId()
        record = self.db.provenance.find_one({'seriesuid':seriesUid})
        return self.factory.fromProvenance(record)

    def knowsSeries(self, image):
        """Whether this file is part of a series for which provenance 
        is available.

        Args:
            image (:class:`.BaseFile`): File for which the series is sought.

        Returns:
            bool: True if provenance is available for this series.
        """
        seriesUid = image.getSeriesId()
        if seriesUid is None:
            return False
        return self.db.provenance.find_one({'seriesuid':seriesUid}) is not None

    def add(self, image):
        """Add the provenance for one file to storage.

        Args:
            image (:class:`.BaseFile`): Image file to store.
        """
        self.db.provenance.insert_one(image.provenance)

    def update(self, image):
        """Save changed provenance for this file..

        Args:
            image (:class:`.BaseFile`): Image file that has changed.
        """
        self.db.provenance.update({'location':image.location.toString()}, 
            image.provenance)

    def all(self):
        """Retrieve all known provenance from storage.

        Returns:
            list: List of provenance for known files.
        """
        records = self.db.provenance.find()
        return [self.factory.fromProvenance(record) for record in records]


    def bySubject(self, subject):
        """Get the provenance for all files of a given participant. 

        Args:
            subject (str): The name or other ID string.

        Returns:
            list: List of provenance for known files imaging this subject.
        """
        records = self.db.provenance.find({'subject':subject})
        return [self.factory.fromProvenance(record) for record in records]

    def byApproval(self, approvalStatus):
        records = self.db.provenance.find({'approval':approvalStatus})
        return [self.factory.fromProvenance(record) for record in records]

    def updateApproval(self, locationString, approvalStatus):
        self.db.provenance.update({'location':locationString}, 
            {'$set': {'approval': approvalStatus}})

    def latest(self):
        records = self.db.provenance.find().sort('added', -1).limit(20)
        return [self.factory.fromProvenance(record) for record in records]

    def statistics(self):
        grps = self.db.provenance.aggregate(
           [{'$group':
                 {
                   '_id': None,
                   'totalsize': { '$sum': '$size' },
                   'count': { '$sum': 1 }
                 }
            }])
        groups = list(grps)
        if len(list(groups)) == 0:
            return {'count':0}
        return list(groups)[0]

    def byId(self, uid):
        record = self.db.provenance.find_one({'id':uid})
        return self.factory.fromProvenance(record)






import os, ConfigParser


class Configuration(object):
    """General settings for niprov.

    Individual settings are documented as follows;

    **setting** *= default_value*
        *type* - Explanation.

    The settings can be changed in the configuration file, or in code.

    All settings:
    """
    database_type = 'file'
    """str: Type of backend in which to store provenance. Currently only 'file'
    """

    database_url = '~/provenance.json'
    """str: URL of the database. If ``database-type`` is ``file``, this is the 
    path to the file."""

    dryrun = False
    """bool: Do not execute commands or make lasting changes to the 
    provenance database."""

    verbose = False
    """bool: Output extra information."""

    discover_file_extensions = ['.PAR','.dcm','.fif','.cnt']
    """list: Discover uses this to determine which files to include. 
    Not strictly extensions, can be any string that appears in the file name. 
    Use comma's to separate items."""

    def __init__(self, configFilePath='~/niprov.cfg'):
        configFilePath = os.path.expanduser(configFilePath)
        if os.path.isfile(configFilePath):
            keys = [k for k in dir(self) if k[0] != '_']
            defaults = {k:getattr(self, k) for k in keys}
            types = {k:type(defaults[k]) for k in keys}
            parser = ConfigParser.SafeConfigParser()
            parser.read(configFilePath)
            for key in keys:
                if not parser.has_option('main', key):
                    val = defaults[key]
                elif types[key] is str:
                    val = parser.get('main', key)
                elif types[key] is bool:
                    val = parser.getboolean('main', key)
                elif types[key] is list:
                    items = parser.get('main', key).split(',')
                    val = [i.strip() for i in items if i is not '']
                setattr(self, key, val)



import unittest
from mock import Mock, patch, call
import os
from tests.ditest import DependencyInjectionTestBase


class AddTests(DependencyInjectionTestBase):

    def setUp(self):
        super(AddTests, self).setUp()
        self.config.dryrun = False
        self.repo.knows.return_value = False
        self.repo.knowsSeries.return_value = False
        self.img = Mock()
        self.lastProvenance = None
        def locAt(loc, provenance):
            self.lastProvenance = provenance
            self.lastPath = loc
            return self.img
        self.fileFactory.locatedAt.side_effect = locAt
        patcher = patch('niprov.adding.datetime')
        self.datetime = patcher.start()
        self.addCleanup(patcher.stop)

    def add(self, path, **kwargs):
        from niprov.adding import add
        return add(path, dependencies=self.dependencies, **kwargs)

    def assertNotCalledWith(self, m, *args, **kwargs):
        c = call(*args, **kwargs)
        assert c not in m.call_args_list, "Unexpectedly found call: "+str(c)

    def test_Returns_provenance_and_status(self):
        new = '/p/f2'
        (image, status) = self.add(new)
        self.assertEqual(image, self.img)
        self.assertEqual(status, 'new')

    def test_Sets_transient_flag_if_provided(self):
        (provenance, status) = self.add('/p/f1', transient=True)
        self.assertEqual(self.lastProvenance['transient'],True)

    def test_Creates_ImageFile_object_with_factory(self):
        (image, status) = self.add('p/afile.f')
        self.assertIs(self.img, image)

    def test_Calls_inspect(self):
        (provenance, status) = self.add('p/afile.f')
        self.img.inspect.assert_called_with()

    def test_Hands_provenance_to_repository(self):
        (provenance, status) = self.add('p/afile.f')
        self.repo.add.assert_any_call(self.img)

    def test_If_discovers_file_that_is_known_ignore_it(self):
        self.repo.knows.return_value = True
        (provenance, status) = self.add('p/afile.f')
        assert not self.repo.add.called
        self.listener.knownFile.assert_called_with(self.img.path)
        self.assertEqual(status, 'known')

    def test_If_repo_doesnt_know_file_but_knows_series_update_series(self):
        self.repo.knows.return_value = False
        self.repo.knowsSeries.return_value = True
        series = Mock()
        self.repo.getSeries.return_value = series
        (provenance, status) = self.add('p/afile.f')
        series.addFile.assert_called_with(self.img)
        self.repo.update.assert_called_with(series)
        self.listener.fileFoundInSeries.assert_called_with(self.img, series)
        self.assertEqual(status, 'series')

    def test_If_inspect_raises_exceptions_tells_listener_and_doesnt_save(self):
        self.img.inspect.side_effect = IOError
        (provenance, status) = self.add('p/afile.f')
        self.assertNotCalledWith(self.repo.add, self.img)
        self.listener.fileError.assert_called_with(self.img.path)
        self.assertEqual(status, 'failed')

    def test_If_dryrun_doesnt_talk_to_repo_and_status_is_test(self):
        self.config.dryrun = True
        (provenance, status) = self.add('p/afile.f')
        assert not self.repo.add.called
        assert not self.repo.update.called
        assert not self.img.inspect.called
        self.assertEqual(status, 'dryrun')

    def test_accepts_optional_provenance(self):
        (provenance, status) = self.add('p/afile.f', provenance={'fob':'bez'})
        self.assertEqual(self.lastProvenance['fob'],'bez')

    def test_If_file_doesnt_exists_tells_listener_and_doesnt_save_prov(self):
        self.filesys.fileExists.return_value = False
        self.assertRaises(IOError, self.add, 'p/afile.f')

    def test_For_nonexisting_transient_file_behaves_normal(self):
        self.filesys.fileExists.return_value = False
        self.add('p/afile.f', transient=True)

    def test_Doesnt_inspect_transient_files(self):
        self.add('p/afile.f', transient=True)
        assert not self.img.inspect.called

    def test_Adds_timestamp(self):
        (provenance, status) = self.add('p/afile.f')
        self.assertEqual(self.lastProvenance['added'],self.datetime.now())

    def test_Adds_uid(self):
        with patch('niprov.adding.shortuuid') as shortuuid:
            shortuuid.uuid.return_value = 'abcdefghijklmn'
            (provenance, status) = self.add('p/afile.f')
            self.assertEqual(self.lastProvenance['id'],'abcdef')

    def test_Makes_paths_absolute(self):
        (provenance, status) = self.add('p/afile.f')
        self.assertEqual(self.lastPath,
            os.path.abspath('p/afile.f'))





from niprov.dependencies import Dependencies


class BaseFile(object):

    def __init__(self, location, provenance=None, dependencies=Dependencies()):
        self.dependencies = dependencies
        self.listener = dependencies.getListener()
        self.filesystem = dependencies.getFilesystem()
        self.hasher = dependencies.getHasher()
        self.serializer = dependencies.getSerializer()
        self.location = dependencies.getLocationFactory().fromString(location)
        if provenance:
            self.provenance = provenance
        else:
            self.provenance = {}
        self.provenance.update(self.location.toDictionary())
        self.path = self.provenance['path']

    def inspect(self):
        self.provenance['size'] = self.filesystem.getsize(self.path)
        self.provenance['created'] = self.filesystem.getctime(self.path)
        self.provenance['hash'] = self.hasher.digest(self.path)
        return self.provenance

    def attach(self):
        """
        Attach the current provenance to the file by saving it encoded 
        in a small textfile alongside it.

        The resulting file's name is like the file it describes,
        but with the .provenance extension.
        """
        provstr = self.serializer.serialize(self.provenance)
        self.filesystem.write(self.path+'.provenance', provstr)

    def getSeriesId(self):
        pass



import os
from niprov.dependencies import Dependencies
from basefile import BaseFile
from dcm import DicomFile
from parrec import ParrecFile
from fif import FifFile
from niprov.cnt import NeuroscanFile


class FileFactory(object):
    """Creates customized File objects.

    Based on libraries (python packages) installed and the filename, will
    return an object that derives from niprov.basefile.BaseFile.
    """

    formats = {'.par':('nibabel', ParrecFile),
               '.dcm':('dicom', DicomFile),
               '.cnt':(None, NeuroscanFile),
               '.fif':('mne',FifFile)}

    def __init__(self, dependencies=Dependencies()):
        self.libs = dependencies.getLibraries()
        self.listener = dependencies.getListener()
        self.dependencies = dependencies
    
    def locatedAt(self, location, provenance=None):
        """Return an object to represent the image file at the given location.

        If no specific class is available to handle a file with the filename 
        provided, will default to a BaseFile. Similarly, if missing the 
        dependency to deal with the format, will fall back to a regular 
        BaseFile.

        Args:
            location: Path to the file to represent.

        Returns:
            :class:`.BaseFile`: An object that derives from BaseFile
        """
        extension = os.path.splitext(location)[1].lower()
        if extension not in self.formats:
            return BaseFile(location, provenance=provenance, 
                dependencies=self.dependencies)
        elif not self.libs.hasDependency(self.formats[extension][0]):
            self.listener.missingDependencyForImage(
                self.formats[extension][0], location)
            return BaseFile(location, 
                dependencies=self.dependencies)
        return self.formats[extension][1](location, provenance=provenance, 
                dependencies=self.dependencies)

    def fromProvenance(self, provenance):
        return self.locatedAt(provenance['location'], provenance)







import unittest
from mock import Mock, patch
import os


class ConfigurationTests(unittest.TestCase):

    def setUp(self):
        self.ospath = Mock()
        self.parser = Mock()
        self.parser.get.side_effect = lambda s, k: 'nothing'

    def createConfiguration(self, configFilePath=None):
        from niprov.config import Configuration
        with patch('niprov.config.os') as osp:
            osp.path = self.ospath
            with patch('niprov.config.ConfigParser') as ConfigParser:
                osp.path.expanduser.side_effect = lambda p: p.replace('~',
                    'user-home-directory')
                ConfigParser.SafeConfigParser.return_value = self.parser
                if configFilePath:
                    conf = Configuration(configFilePath)
                else:
                    conf = Configuration()
        return conf

    def test_If_no_config_file_uses_defaults(self):
        self.ospath.isfile.return_value = False
        conf = self.createConfiguration()
        self.assertEqual(False, conf.verbose)
        self.assertEqual(False, conf.dryrun)

    def test_If_config_file_reads_it(self):
        self.ospath.isfile.return_value = True
        conf = self.createConfiguration()
        self.parser.read.assert_called_with('user-home-directory/niprov.cfg')

    def test_Can_pass_file_location(self):
        self.ospath.isfile.return_value = True
        conf = self.createConfiguration('~/my/path.cfg')
        self.parser.read.assert_called_with('user-home-directory/my/path.cfg')

    def test_Uses_values_in_file(self):
        self.ospath.isfile.return_value = True
        self.parser.getboolean.side_effect = lambda s, k: True
        self.parser.get.side_effect = lambda s, k: 'mothership'
        conf = self.createConfiguration()
        self.assertEqual(True, conf.verbose)
        self.assertEqual('mothership', conf.database_type)

    def test_Can_deal_with_lists(self):
        self.ospath.isfile.return_value = True
        self.parser.get.side_effect = lambda s, k: 'a1,b2, c3,d4 ,'
        conf = self.createConfiguration()
        self.assertEqual(['a1','b2','c3','d4'], conf.discover_file_extensions)




import os
from operator import itemgetter
from niprov.dependencies import Dependencies


class JsonFile(object):
    """Stores provenance in a local text file encoded as json.
    """

    def __init__(self, dependencies=Dependencies()):
        self.filesys = dependencies.getFilesystem()
        self.json = dependencies.getSerializer()
        self.factory = dependencies.getFileFactory()
        url = dependencies.getConfiguration().database_url
        self.datafile = os.path.expanduser(url)

    def add(self, image):
        """Add the provenance for one file to storage.

        Args:
            image (:class:`.BaseFile`): Image file to store.
        """
        current = self._all()
        current.append(image.provenance)
        jsonstr = self.json.serializeList(current)
        self.filesys.write(self.datafile, jsonstr)

    def update(self, image):
        """Save changed provenance for this file..

        Args:
            image (:class:`.BaseFile`): Image file that has changed.
        """
        current = self._all()
        for r in range(len(current)):
            if current[r]['location'] == image.location.toString():
                current[r] = image.provenance
        jsonstr = self.json.serializeList(current)
        self.filesys.write(self.datafile, jsonstr)

    def all(self):
        """Retrieve all known provenance from storage.

        Returns:
            list: List of provenance for known files.
        """
        records = self._all()
        return [self.factory.fromProvenance(record) for record in records]

    def _all(self):
        try:
            jsonstr = self.filesys.read(self.datafile)
        except IOError:
            return []
        return self.json.deserializeList(jsonstr)

    def knowsByLocation(self, locationString):
        """Whether the file at this path has provenance associated with it.

        Returns:
            bool: True if provenance is available for that path.
        """
        try:
            self.byLocation(locationString)
        except IndexError:
            return False
        return True

    def knows(self, image):
        """Whether this file has provenance associated with it.

        Returns:
            bool: True if provenance is available for this image.
        """
        try:
            self.byLocation(image.path)
        except IndexError:
            return False
        return True

    def knowsSeries(self, image):
        """Whether the series that this file is part of has provenance 
        associated with it.

        Args:
            image (:class:`.BaseFile`): File for which the series is sought.

        Returns:
            bool: True if provenance is available for this series.
        """
        try:
            self.getSeries(image)
        except IndexError:
            return False
        return True

    def byLocation(self, locationString):
        """Get the provenance for a file at the given location. 

        In the case of a dicom series, this returns the provenance for the 
        series.

        Args:
            locationString (str): Location of the image file.

        Returns:
            dict: Provenance for one image file.
        """
        for record in self._all():
            if record['location'] == locationString:
                return self.factory.fromProvenance(record)
            elif 'filesInSeries' in record and locationString in record['filesInSeries']:
                return self.factory.fromProvenance(record)
        else:
            raise IndexError('No file with that path known.')

    def bySubject(self, subject):
        """Get the provenance for all files of a given participant. 

        Args:
            subject (str): The name or other ID string.

        Returns:
            list: List of provenance for known files imaging this subject.
        """
        all = self._all()
        recordsWithSubject = [f for f in all if 'subject' in f]
        records = [f for f in recordsWithSubject if f['subject']==subject]
        return [self.factory.fromProvenance(record) for record in records]

    def getSeries(self, image):
        """Get the object that carries provenance for the series that the image 
        passed is in. 

        Args:
            image (:class:`.DicomFile`): File that is part of a series.

        Returns:
            :class:`.DicomFile`: Image object that caries provenance for the series.
        """
        if image.getSeriesId() is None:
            raise IndexError('Image has no series id.')
        seriesId = image.getSeriesId()
        for record in self._all():
            if 'seriesuid' in record and record['seriesuid'] == seriesId:
                return self.factory.fromProvenance(record)
        else:
            raise IndexError('No provenance record for that series.')

    def byApproval(self, approvalStatus):
        allRecords = self._all()
        matches = []
        for prov in allRecords:
            if 'approval' in prov:
                if prov['approval'] == approvalStatus:
                    matches.append(prov)
        return [self.factory.fromProvenance(r) for r in matches]

    def updateApproval(self, fpath, approvalStatus):
        img = self.byLocation(fpath)
        img.provenance['approval'] = approvalStatus
        self.update(img)

    def latest(self, n=20):
        allRecords = self._all()
        sortedRecords = sorted(allRecords, key=itemgetter('added'), reverse=True)
        records = sortedRecords[:n]
        return [self.factory.fromProvenance(record) for record in records]

    def statistics(self):
        stats = {}
        allRecords = self._all()
        stats['count'] = len(allRecords)
        stats['totalsize'] = sum([r['size'] for r in allRecords])
        return stats

    def byId(self, uid):
        for record in self._all():
            if record['id'] == uid:
                return self.factory.fromProvenance(record)
        else:
            raise IndexError('No file with that path known.')

       




import unittest
from mock import Mock, patch, sentinel


class MongoRepoTests(unittest.TestCase):

    def setUp(self):
        self.dependencies = Mock()
        self.config = Mock()
        self.factory = Mock()
        self.dependencies.getConfiguration.return_value = self.config
        self.dependencies.getFileFactory.return_value = self.factory
        self.db = Mock()

    def setupRepo(self):
        from niprov.mongo import MongoRepository
        with patch('niprov.mongo.pymongo') as pymongo:
            self.repo = MongoRepository(dependencies=self.dependencies)
        self.repo.db = self.db

    def test_Connection(self):
        from niprov.mongo import MongoRepository
        with patch('niprov.mongo.pymongo') as pymongo:
            repo = MongoRepository(dependencies=self.dependencies)
        pymongo.MongoClient.assert_called_with(self.config.database_url)
        self.assertEqual(pymongo.MongoClient().get_default_database(), repo.db)

    def test_knowsByLocation(self):
        self.setupRepo()
        p = '/p/f1'
        self.db.provenance.find_one.return_value = None
        self.assertFalse(self.repo.knowsByLocation(p))
        self.db.provenance.find_one.assert_called_with({'location':p})
        self.db.provenance.find_one.return_value = 1
        self.assertTrue(self.repo.knowsByLocation(p))

    def test_byLocation_returns_img_from_record_with_path(self):
        self.setupRepo()
        p = '/p/f1'
        out = self.repo.byLocation(p)
        self.db.provenance.find_one.assert_called_with({'location':p})
        self.factory.fromProvenance.assert_called_with(
            self.db.provenance.find_one())
        self.assertEqual(self.factory.fromProvenance(), out)

    def test_getSeries(self):
        self.setupRepo()
        img = Mock()
        out = self.repo.getSeries(img)
        self.db.provenance.find_one.assert_called_with(
            {'seriesuid':img.getSeriesId()})
        self.factory.fromProvenance.assert_called_with(
            self.db.provenance.find_one())
        self.assertEqual(self.factory.fromProvenance(), out)

    def test_knowsSeries_returns_False_if_no_series_id(self):
        self.setupRepo()
        img = Mock()
        img.getSeriesId.return_value = None
        self.assertFalse(self.repo.knowsSeries(img))

    def test_knowsSeries(self):
        self.setupRepo()
        img = Mock()
        self.assertTrue(self.repo.knowsSeries(img))
        self.db.provenance.find_one.return_value = None
        self.assertFalse(self.repo.knowsSeries(img))

    def test_Add(self):
        self.setupRepo()
        img = Mock()
        self.repo.add(img)
        self.db.provenance.insert_one.assert_called_with(img.provenance)

    def test_update(self):
        self.setupRepo()
        img = Mock()
        self.repo.update(img)
        self.db.provenance.update.assert_called_with(
            {'location':img.location.toString()}, img.provenance)

    def test_all(self):
        self.factory.fromProvenance.side_effect = lambda p: 'img_'+p
        self.db.provenance.find.return_value = ['p1', 'p2']
        self.setupRepo()
        out = self.repo.all()
        self.db.provenance.find.assert_called_with()
        self.factory.fromProvenance.assert_any_call('p1')
        self.factory.fromProvenance.assert_any_call('p2')
        self.assertEqual(['img_p1', 'img_p2'], out)

    def test_bySubject(self):
        self.factory.fromProvenance.side_effect = lambda p: 'img_'+p
        self.db.provenance.find.return_value = ['p1', 'p2']
        self.setupRepo()
        s = 'Brambo'
        out = self.repo.bySubject(s)
        self.db.provenance.find.assert_called_with({'subject':s})
        self.factory.fromProvenance.assert_any_call('p1')
        self.factory.fromProvenance.assert_any_call('p2')
        self.assertEqual(['img_p1', 'img_p2'], out)

    def test_byApproval(self):
        self.factory.fromProvenance.side_effect = lambda p: 'img_'+p
        self.db.provenance.find.return_value = ['p1', 'p2']
        self.setupRepo()
        a = 'AOk'
        out = self.repo.byApproval(a)
        self.db.provenance.find.assert_called_with({'approval':a})
        self.factory.fromProvenance.assert_any_call('p1')
        self.factory.fromProvenance.assert_any_call('p2')
        self.assertEqual(['img_p1', 'img_p2'], out)

    def test_updateApproval(self):
        self.setupRepo()
        img = Mock()
        p = '/p/f1'
        newStatus = 'oh-oh'
        self.repo.updateApproval(p, newStatus)
        self.db.provenance.update.assert_called_with(
            {'location':p}, {'$set': {'approval': newStatus}})

    def test_latest(self):
        self.factory.fromProvenance.side_effect = lambda p: 'img_'+p
        self.db.provenance.find.return_value.sort.return_value.limit.return_value = ['px','py']
        self.setupRepo()
        out = self.repo.latest()
        self.db.provenance.find.assert_called_with()
        self.db.provenance.find().sort.assert_called_with('added', -1)
        self.db.provenance.find().sort().limit.assert_called_with(20)
        self.factory.fromProvenance.assert_any_call('px')
        self.factory.fromProvenance.assert_any_call('py')
        self.assertEqual(['img_px', 'img_py'], out)

    def test_statistics(self):
        self.db.provenance.aggregate.return_value = [sentinel.stats,]
        self.setupRepo()
        out = self.repo.statistics()
        self.db.provenance.aggregate.assert_called_with(
           [{'$group':
                 {
                   '_id': None,
                   'totalsize': { '$sum': '$size' },
                   'count': { '$sum': 1 }
                 }
            }])
        self.assertEqual(sentinel.stats, out)

    def test_statistics_if_no_records(self):
        self.db.provenance.aggregate.return_value = []
        self.setupRepo()
        out = self.repo.statistics()
        self.assertEqual({'count':0}, out)

    def test_byId(self):
        self.setupRepo()
        ID = 'abc123'
        out = self.repo.byId(ID)
        self.db.provenance.find_one.assert_called_with({'id':ID})
        self.factory.fromProvenance.assert_called_with(
            self.db.provenance.find_one())
        self.assertEqual(self.factory.fromProvenance(), out)





from niprov.dependencies import Dependencies
from niprov.config import Configuration
import niprov


class Context(object):

    def __init__(self):
        self.config = Configuration()
        self.deps = Dependencies(self.config)

    def add(self, filepath, transient=False):
        """See :py:mod:`niprov.config`  """
        return niprov.adding.add(filepath, transient, dependencies=self.deps)

    def discover(self, root):
        """See :py:mod:`niprov.discovery`  """
        return niprov.discovery.discover(root, dependencies=self.deps)

    def inspect(self, location):
        """See :py:mod:`niprov.inspection`  """
        return niprov.inspection.inspect(location, dependencies=self.deps)

    def log(self, *args, **kwargs):
        """See :py:mod:`niprov.logging`  """
        return niprov.logging.log(*args, dependencies=self.deps, **kwargs)

    def markForApproval(self, files):
        """See :py:mod:`niprov.approval`  """
        return niprov.approval.markForApproval(files, dependencies=self.deps)

    def markedForApproval(self):
        """See :py:mod:`niprov.approval`  """
        return niprov.approval.markedForApproval(dependencies=self.deps)

    def approve(self, filepath):
        """See :py:mod:`niprov.approval`  """
        return niprov.approval.approve(filepath, dependencies=self.deps)

    def selectApproved(self, files):
        """See :py:mod:`niprov.approval`  """
        return niprov.approval.selectApproved(files, dependencies=self.deps)

    def renameDicoms(self, dicomdir):
        """See :py:mod:`niprov.renaming`  """
        return niprov.renaming.renameDicoms(dicomdir, dependencies=self.deps)

    def record(self, *args, **kwargs):
        """See :py:mod:`niprov.recording`  """
        return niprov.recording.record(*args, dependencies=self.deps, **kwargs)

    def report(self, *args, **kwargs):
        """See :py:mod:`niprov.reporting`  """
        return niprov.reporting.report(*args, dependencies=self.deps, **kwargs)



import unittest
from mock import Mock
from tests.ditest import DependencyInjectionTestBase


class ApprovalTests(DependencyInjectionTestBase):

    def setUp(self):
        super(ApprovalTests, self).setUp()

    def test_markForApproval_tells_repo_to_set_approval_to_pending(self):
        import niprov
        niprov.markForApproval(['f1','f2'], dependencies=self.dependencies)
        self.repo.updateApproval.assert_any_call('f1','pending')
        self.repo.updateApproval.assert_any_call('f2','pending')

    def test_Approve_tells_repo_to_set_approval_to_granted(self):
        import niprov
        niprov.approve('fx12', dependencies=self.dependencies)
        self.repo.updateApproval.assert_any_call('fx12','granted')

    def test_markedForApproval_lists_files_marked(self):
        import niprov
        markedFiles = niprov.markedForApproval(dependencies=self.dependencies)
        self.repo.byApproval.assert_called_with('pending')
        self.assertEqual(self.repo.byApproval(), markedFiles)

    def test_markedForApproval_tells_listener_about_files(self):
        import niprov
        markedFiles = niprov.markedForApproval(dependencies=self.dependencies)
        self.listener.filesMarkedForApproval.assert_called_with(
            markedFiles)

    def test_selectApproved(self):
        import niprov
        self.locationFactory.completeString.side_effect = lambda p: p
        a1 = self.mockImg()
        a1.provenance['approval'] = 'granted'
        a2 = self.mockImg()
        a2.provenance['approval'] = 'granted'
        b1 = self.mockImg()
        c1 = self.mockImg()
        c1.provenance['approval'] = 'pending'
        self.repo.byLocation.side_effect = lambda p: {'a1':a1,'b1':b1,'a2':a2,'c1':c1}[p]
        selected = niprov.selectApproved(['a1','b1','a2','c1'], 
            dependencies=self.dependencies)
        self.locationFactory.completeString.assert_any_call('a1')
        self.locationFactory.completeString.assert_any_call('b1')
        self.locationFactory.completeString.assert_any_call('a2')
        self.locationFactory.completeString.assert_any_call('c1')
        self.assertEqual(selected, [a1.path, a2.path])

    def mockImg(self):
        img = Mock()
        img.provenance = {}
        return img






class Repository(object):

    def byLocation(self, locationString):
        """Get the provenance for a file at the given location. 

        In the case of a dicom series, this returns the provenance for the 
        series.

        Args:
            locationString (str): Location of the image file.

        Returns:
            dict: Provenance for one image file.
        """

    def knowsByLocation(self, locationString):
        """Whether the file at this location has provenance associated with it.

        Returns:
            bool: True if provenance is available for that path.
        """

    def knows(self, image):
        """Whether this file has provenance associated with it.

        Returns:
            bool: True if provenance is available for this image.
        """

    def getSeries(self, image):
        """Get the object that carries provenance for the series that the image 
        passed is in. 

        Args:
            image (:class:`.DicomFile`): File that is part of a series.

        Returns:
            :class:`.DicomFile`: Image object that caries provenance for the series.
        """

    def knowsSeries(self, image):
        """Whether this file is part of a series for which provenance 
        is available.

        Args:
            image (:class:`.BaseFile`): File for which the series is sought.

        Returns:
            bool: True if provenance is available for this series.
        """

    def add(self, image):
        """Add the provenance for one file to storage.

        Args:
            image (:class:`.BaseFile`): Image file to store.
        """

    def update(self, image):
        """Save changed provenance for this file..

        Args:
            image (:class:`.BaseFile`): Image file that has changed.
        """

    def all(self):
        """Retrieve all known provenance from storage.

        Returns:
            list: List of provenance for known files.
        """

    def bySubject(self, subject):
        """Get the provenance for all files of a given participant. 

        Args:
            subject (str): The name or other ID string.

        Returns:
            list: List of provenance for known files imaging this subject.
        """

    def byApproval(self, approvalStatus):
        """"""

    def updateApproval(self, locationString, approvalStatus):
        """"""

    def latest(self, n=20):
        """Get the images that have been registered last. 

        Args:
            n (int): The number of files to retrieve. Defaults to 20.

        Returns:
            list: List of BaseFile objects.
        """

    def byId(self, uid):
        """Get the provenance for a file with the given id. 

        Args:
            uid (str): Unique id for the file.

        Returns:
            BaseFile: File with the given id.
        """



import unittest
from mock import Mock, sentinel, patch
from datetime import datetime as dt
from tests.ditest import DependencyInjectionTestBase


class LoggingTests(DependencyInjectionTestBase):

    def setUp(self):
        super(LoggingTests, self).setUp()
        self.repo.knowsByLocation.return_value = True
        self.opts = Mock()
        self.opts.dryrun = False
        img = Mock()
        img.provenance = {'acquired':dt.now(),'subject':'JD', 'protocol':'X'}
        self.repo.byLocation.return_value = img
        self.newimg = Mock()
        self.provenancesCreated = []
        def wrapProv(location,transient,provenance,dependencies):
            self.provenancesCreated.append(provenance)
            return self.newimg
        self.dependencies.reconfigureOrGetConfiguration.return_value = self.opts
        patcher = patch('niprov.logging.add')
        self.add = patcher.start()
        self.add.side_effect = wrapProv
        self.addCleanup(patcher.stop)

    def log(self, *args, **kwargs):
        from niprov.logging import log
        return log(*args, dependencies=self.dependencies, opts=self.opts, 
            **kwargs)

    def test_Returns_img(self):
        parents = ['/p/f1']
        new = '/p/f2'
        trans = 'Something cool'
        out = self.log(new, trans, parents)
        self.assertEqual(out, self.newimg)

    def test_Copies_fields_from_known_parent(self):
        self.locationFactory.completeString.side_effect = lambda p: p
        parent = '/p/f1'
        parents = [parent]
        parentProv = {'acquired':dt.now(),'subject':'JB','protocol':'T3'}
        parentImg = Mock()
        parentImg.provenance = parentProv
        self.repo.byLocation.side_effect = lambda x: {parent:parentImg}[x]
        self.log('new', 'trans', parents)
        self.assertEqual(self.provenancesCreated[0]['acquired'], parentProv['acquired'])
        self.assertEqual(self.provenancesCreated[0]['subject'], parentProv['subject'])
        self.assertEqual(self.provenancesCreated[0]['protocol'], parentProv['protocol'])

    def test_Adds_code_or_logtext(self):
        self.log('new', 'trans', 'old', code='abc', logtext='def')
        self.assertEqual(self.provenancesCreated[0]['code'],'abc')
        self.assertEqual(self.provenancesCreated[0]['logtext'],'def')

    def test_Script_added_to_provenance(self):
        parents = ['/p/f1']
        new = '/p/f2'
        trans = 'Something cool'
        script = '/p/test.py'
        self.log(new, trans, parents, script=script)
        self.assertEqual(self.provenancesCreated[0]['script'], script)

    def test_Accepts_and_processes_custom_provenance(self):
        parents = ['/p/f1']
        new = '/p/f2'
        trans = 'Something cool'
        p = {'akey':'avalue'}
        self.log(new, trans, parents, provenance=p)
        self.assertEqual(self.provenancesCreated[0]['akey'], 'avalue')

    def test_Doesnt_complain_if_parent_is_missing_basic_fields(self):
        img = Mock()
        img.provenance = {'acquired':dt.now()} #missing subject
        self.repo.byLocation.return_value = img
        provenance = self.log('new', 'trans', ['/p/f1parent'])
        self.assertNotIn('subject', self.provenancesCreated[0])

    def test_Calls_reconfigureOrGetConfiguration_on_dependencies(self):
        outOpts = Mock()
        outOpts.dryrun = True
        self.dependencies.reconfigureOrGetConfiguration.return_value = outOpts
        provenance = self.log(['/p/f1'], 'bla', ['/p/f2'], transient=True)
        self.dependencies.reconfigureOrGetConfiguration.assert_called_with(
            self.opts)

    def test_Can_pass_multiple_new_files(self):
        parents = ['p1','p2']
        new = ['/p/f2','/p/f3']
        trans = 'Something cool'
        self.locationFactory.completeString.side_effect = lambda p: 'l:'+p
        self.log(new, trans, parents)
        self.assertEqual(self.provenancesCreated[0]['parents'], ['l:p1','l:p2'])
        self.assertEqual(self.provenancesCreated[1]['parents'], ['l:p1','l:p2'])

    def test_Notifies_listener_and_exits_if_parent_unknown(self):
        self.repo.knowsByLocation.return_value = False
        parent = '/p/f1'
        parents = [parent]
        provenance = self.log('new', 'trans', parents)
        self.listener.unknownFile.assert_called_with(parent)
        assert not self.add.called

    def test_Finds_parent_provenance_using_completedString(self):
        self.locationFactory.completeString.side_effect = lambda p: 'l:'+p
        parent = '/p/f1'
        parents = [parent]
        provenance = self.log('new', 'trans', parents)
        self.repo.knowsByLocation.assert_called_with(self.provenancesCreated[0]['parents'][0])
        self.repo.byLocation.assert_called_with(self.provenancesCreated[0]['parents'][0])
        




import unittest
from mock import Mock, patch
from tests.ditest import DependencyInjectionTestBase


class ViewTests(DependencyInjectionTestBase):

    def setUp(self):
        super(ViewTests, self).setUp()
        self.config.dryrun = False
        self.img = Mock()
        self.request = Mock()
        self.request.dependencies = self.dependencies

    def test_latest(self):
        import niprov.views
        out = niprov.views.latest(self.request)
        self.assertEqual(self.repo.latest(), out['images'])

    def test_by_id(self):
        import niprov.views
        self.request.matchdict = {'id':'1a2b3c'}
        out = niprov.views.short(self.request)
        self.repo.byId.assert_called_with('1a2b3c')
        self.assertEqual(self.repo.byId(), out['image'])

    def test_by_full_location(self):
        import niprov.views
        self.request.matchdict = {'host':'her','path':('a','b','c')}
        out = niprov.views.location(self.request)
        self.repo.byLocation.assert_called_with('her:/a/b/c')
        self.assertEqual(self.repo.byLocation(), out['image'])

    def test_stats(self):
        import niprov.views
        out = niprov.views.stats(self.request)
        self.assertEqual(self.repo.statistics(), out['stats'])

    def test_by_subject(self):
        import niprov.views
        self.request.matchdict = {'subject':'janedoe'}
        out = niprov.views.subject(self.request)
        self.repo.bySubject.assert_called_with('janedoe')
        self.assertEqual(self.repo.bySubject(), out['images'])








from mock import Mock
from tests.ditest import DependencyInjectionTestBase
import datetime, random


class JsonFileTest(DependencyInjectionTestBase):

    def setUp(self):
        super(JsonFileTest, self).setUp()
        self.dependencies.getConfiguration().database_url = ''

    def test_Update(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        repo._all = Mock()
        repo._all.return_value = [{'location':'1','path':'a'},
            {'location':'2','path':'b'}]
        image = Mock()
        image.location.toString.return_value = '2'
        image.provenance = {'foo':'bar'}
        repo.update(image)
        self.serializer.serializeList.assert_called_with(
            [{'location':'1','path':'a'},{'foo':'bar'}])
        self.filesys.write.assert_called_with(repo.datafile, 
            self.serializer.serializeList())

    def test_Add(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        repo._all = Mock()
        repo._all.return_value = [{'location':'1','path':'a'}]
        image = Mock()
        image.provenance = {'foo':'bar'}
        repo.add(image)
        self.serializer.serializeList.assert_called_with(
            [{'location':'1','path':'a'},{'foo':'bar'}])
        self.filesys.write.assert_called_with(repo.datafile, 
            self.serializer.serializeList())

    def test_knowsByLocation(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        repo.byLocation = Mock()
        self.assertTrue(repo.knowsByLocation(''))
        repo.byLocation.side_effect = IndexError
        self.assertFalse(repo.knowsByLocation(''))

    def test_byLocation(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        repo._all = Mock()
        repo._all.return_value = [{'location':'1','path':'a'},
            {'location':'2','path':'b'}]
        out = repo.byLocation('1')
        self.fileFactory.fromProvenance.assert_called_with(
            {'location':'1','path':'a'})
        self.assertEqual(self.fileFactory.fromProvenance(), out)

    def test_byLocation_works_for_file_in_series(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        repo._all = Mock()
        repo._all.return_value = [{'location':'1','path':'a'},
            {'location':'3','filesInSeries':['boo','bah']}]
        out = repo.byLocation('boo')
        self.fileFactory.fromProvenance.assert_called_with(
            {'location':'3','filesInSeries':['boo','bah']})
        self.assertEqual(self.fileFactory.fromProvenance(), out)

    def test_updateApproval(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        img = Mock()
        img.provenance = {'fiz':'baf','approval':'notsure'}
        def assertion(img):
            self.assertEqual({'fiz':'baf','approval':'excellent!'}, img.provenance)
        repo.byLocation = Mock()
        repo.byLocation.return_value = img
        repo.update = Mock()
        repo.update.side_effect = assertion
        repo.updateApproval('/p/f1','excellent!')

    def test_bySubject(self):
        self.fileFactory.fromProvenance.side_effect = lambda p: 'img_'+p['a']
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        repo._all = Mock()
        repo._all.return_value = [{'subject':'john','a':'b'},
            {'subject':'tim','a':'d'},{'subject':'john','a':'f'}]
        out = repo.bySubject('john')
        self.fileFactory.fromProvenance.assert_any_call({'subject':'john','a':'b'})
        self.fileFactory.fromProvenance.assert_any_call({'subject':'john','a':'f'})
        self.assertEqual(['img_b', 'img_f'], out)

    def test_byApproval(self):
        self.fileFactory.fromProvenance.side_effect = lambda p: 'img_'+p['a']
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        repo._all = Mock()
        repo._all.return_value = [{'approval':'y','a':'b'},
            {'approval':'x','a':'d'},{'approval':'x','a':'f'}]
        out = repo.byApproval('x')
        self.fileFactory.fromProvenance.assert_any_call({'approval':'x','a':'d'})
        self.fileFactory.fromProvenance.assert_any_call({'approval':'x','a':'f'})
        self.assertEqual(['img_d', 'img_f'], out)

    def test_latest(self):
        self.fileFactory.fromProvenance.side_effect = lambda p: 'img_'+str(p['added'])
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        d1 = datetime.datetime(1982, 1, 5)
        d2 = datetime.datetime(1982, 2, 5)
        d3 = datetime.datetime(1982, 3, 5)
        d4 = datetime.datetime(1982, 4, 5)
        d5 = datetime.datetime(1982, 5, 5)
        repo._all = Mock()
        repo._all.return_value = [{'added':d, 'n':i+1} for i,d in enumerate([d1,d2,d3,d4,d5])]
        out = repo.latest(3)
        self.fileFactory.fromProvenance.assert_any_call({'added':d5, 'n':5})
        self.fileFactory.fromProvenance.assert_any_call({'added':d4, 'n':4})
        self.fileFactory.fromProvenance.assert_any_call({'added':d3, 'n':3})
        self.assertEqual(['img_1982-05-05 00:00:00', 'img_1982-04-05 00:00:00',
            'img_1982-03-05 00:00:00'], out)

    def test_stats(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        repo._all = Mock()
        records = [{},{},{},{},{},{},{},{},{},{},{}]
        totalsize = 0
        for r in records:
            r['size'] = random.randint(1,1000)
            totalsize += r['size']
        repo._all.return_value = records
        out = repo.statistics()
        self.assertEqual(11, out['count'])
        self.assertEqual(totalsize, out['totalsize'])
        
    def test_byId(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        repo._all = Mock()
        repo._all.return_value = [{'id':'1','path':'a'},
            {'id':'2','path':'b'}]
        out = repo.byId('2')
        self.fileFactory.fromProvenance.assert_called_with(
            {'id':'2','path':'b'})
        self.assertEqual(self.fileFactory.fromProvenance(), out)

    def test_bySubject_doesnt_balk_if_no_subject_field(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(self.dependencies)
        repo._all = Mock()
        repo._all.return_value = [{'subject':'john','a':'b'},
            {'a':'d'},{'subject':'john','a':'f'}]
        out = repo.bySubject('john')




import unittest, os
from pymongo import MongoClient

URL = 'mongodb://niprov-admin:{0}@ds041571.mongolab.com:41571/niprov'
PASSWORD = 'uU0mpDXtQXeL6wku'


class MongoTests(unittest.TestCase):

    def setUp(self):
        from niprov import Context
        self.provenance = Context()
        self.fullURL = URL.format(PASSWORD)
        self.provenance.config.database_type = 'MongoDB'
        self.provenance.config.database_url = self.fullURL

    def tearDown(self):
        client = MongoClient(self.fullURL)
        client.get_default_database().provenance.drop()

    def test_Something(self):
        self.provenance.discover('testdata')
        testfpath = os.path.abspath('testdata/parrec/T1.PAR')
        img = self.provenance.report(forFile=testfpath)
        self.assertEqual(img.provenance['subject'], '05aug14test')

    def test_Stats(self):
        self.provenance.discover('testdata')
        repository = self.provenance.deps.getRepository()
        print(repository.statistics())






#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.dependencies import Dependencies
from niprov.stdout import StandardOutputExporter
from niprov.directexporter import DirectExporter


class ExportFactory(object):

    def __init__(self, dependencies=Dependencies()):
        self.dependencies = dependencies

    def createExporter(self, medium, form):
        """
        Return an object that can publish provenance in a specific medium.

        Args:
            medium (str): One of 'stdout' or None.
            form (str): One of 'narrative' or None
        """
        if medium is None:
            return DirectExporter(form)
        elif medium == 'stdout':
            return StandardOutputExporter(form)
        else:
            raise ValueError('Unknown medium: '+str(medium))




import unittest
import os, shutil


class ContextApiTests(unittest.TestCase):

    def setUp(self):
        self.dbpath = os.path.expanduser(os.path.join('~','provenance_test.json'))
        if os.path.exists(self.dbpath):
            os.remove(self.dbpath)
        os.mkdir('temp')
        from niprov import Context
        self.provenance = Context()
        self.provenance.config.database_type = 'file'
        self.provenance.config.database_url = self.dbpath

    def tearDown(self):
        shutil.rmtree('temp')

    def touch(self, path):
        with open(path,'w') as tempfile:
            tempfile.write('0')

    def test_Log(self):
        self.provenance.discover('testdata')
        newfile = 'temp/smoothed.test'
        self.touch(newfile)
        parent = os.path.abspath('testdata/eeg/stub.cnt')
        self.provenance.log(newfile, 'test', parent)
        testfpath = os.path.abspath(newfile)
        img = self.provenance.report(forFile=testfpath)
        self.assertEqual(img.provenance['subject'], 'Jane Doe')
        self.assertEqual(img.provenance['size'], os.path.getsize(newfile))




import unittest
from mock import Mock


class DependencyInjectionTestBase(unittest.TestCase):

    def setUp(self):
        self.config = Mock()
        self.listener = Mock()
        self.hasher = Mock()
        self.filesys = Mock()
        self.json = Mock()
        self.repo = Mock()
        self.exporter = Mock()
        self.exportFactory = Mock()
        self.exportFactory.createExporter.return_value = self.exporter
        self.fileFactory = Mock()
        self.serializer = Mock()
        self.location = Mock()
        self.clock = Mock()
        self.locationFactory = Mock()
        self.locationFactory.fromString.return_value = self.location
        self.dependencies = Mock()
        self.dependencies.getLocationFactory.return_value = self.locationFactory
        self.dependencies.getListener.return_value = self.listener
        self.dependencies.getHasher.return_value = self.hasher
        self.dependencies.getFilesystem.return_value = self.filesys
        self.dependencies.getSerializer.return_value = self.json
        self.dependencies.getRepository.return_value = self.repo
        self.dependencies.getFileFactory.return_value = self.fileFactory
        self.dependencies.getSerializer.return_value = self.serializer
        self.dependencies.getExportFactory.return_value = self.exportFactory
        self.dependencies.getConfiguration.return_value = self.config
        self.dependencies.getClock.return_value = self.clock





from mock import Mock, patch
from tests.ditest import DependencyInjectionTestBase


class LocationTests(DependencyInjectionTestBase):

    def setUp(self):
        super(LocationTests, self).setUp()

    def test_LocationFactory_fromString(self):
        from niprov.locationfactory import LocationFactory
        with patch('niprov.locationfactory.Location') as Location:
            factory = LocationFactory()
            out = factory.fromString('abc')
            Location.assert_called_with('abc')
            self.assertEqual(Location(), out)

    def test_Location_toDictionary(self):
        from niprov.location import Location
        loc = Location('HAL:/p/n1.f')
        expected = {'path':'/p/n1.f','hostname':'HAL','location':'HAL:/p/n1.f'}
        self.assertEqual(expected, loc.toDictionary())

    def test_Location_by_default_fills_in_local_hostname(self):
        from niprov.location import Location
        with patch('niprov.location.socket') as socket:
            loc = Location('/p/n1.f')
            self.assertEqual(socket.gethostname(), loc.hostname)

    def test_Location_parses_locationString(self):
        from niprov.location import Location
        with patch('niprov.location.socket') as socket:
            loc = Location('HAL:/p/n1.f')
            self.assertEqual('HAL', loc.hostname)
            self.assertEqual('/p/n1.f', loc.path)

    def test_Location_stringifies_to_full_locationString(self):
        from niprov.location import Location
        with patch('niprov.location.socket') as socket:
            loc = Location('HAL:/p/n1.f')
            self.assertEqual('HAL:/p/n1.f', str(loc))

    def test_LocationFactory_completeString(self):
        from niprov.locationfactory import LocationFactory
        with patch('niprov.locationfactory.Location') as Location:
            factory = LocationFactory()
            factory.fromString = Mock()
            outstr = factory.completeString('/j/k/l')
            factory.fromString.assert_called_with('/j/k/l')
            self.assertEqual(str(factory.fromString()), outstr)




import unittest
from mock import Mock
from tests.ditest import DependencyInjectionTestBase

class ReportingTests(DependencyInjectionTestBase):

    def setUp(self):
        super(ReportingTests, self).setUp()

    def report(self, *args, **kwargs):
        import niprov
        return niprov.report(*args, dependencies=self.dependencies, **kwargs)

    def test_Without_specifics_returns_all_files(self):
        out = self.report()
        self.exporter.export.assert_called_with(self.repo.all())
        self.assertEqual(out, self.exporter.export())

    def test_Can_report_on_one_file_specifically(self):
        out = self.report(forFile='afile.f')
        self.exporter.export.assert_called_with(self.repo.byLocation('afile.f'))
        self.assertEqual(out, self.exporter.export())

    def test_Completes_locationString_forFile(self):
        out = self.report(forFile='afile.f')
        self.locationFactory.completeString.assert_any_call('afile.f')
        self.repo.byLocation.assert_called_with(self.locationFactory.completeString())

    def test_Can_report_on_all_files_for_subject(self):
        out = self.report(forSubject='Jane Doe')
        self.exporter.export.assert_called_with(
            self.repo.bySubject('Jane Doe'))
        self.assertEqual(out, self.exporter.export())

    def test_Obtains_exporter_from_factory(self):
        self.report()
        self.exportFactory.createExporter.assert_any_call(None, None)
        self.report(medium='html')
        self.exportFactory.createExporter.assert_any_call('html', None)
        self.report(form='narrative')
        self.exportFactory.createExporter.assert_any_call(None, 'narrative')

    def test_Passes_provenance_to_exporter(self):
        self.report()
        self.exportFactory.createExporter().export.assert_any_call(self.repo.all())
        self.report(forSubject='Jane')
        self.exportFactory.createExporter().export.assert_any_call(self.repo.bySubject())

    def test_Passes_single_item_of_provenance_to_exporter(self):
        self.report(forFile='xyz')
        self.exportFactory.createExporter().export.assert_any_call(
            self.repo.byLocation('xyz'))

    def test_If_file_unknown_tells_listener(self):
        self.locationFactory.completeString.side_effect = lambda p: p
        self.repo.knowsByLocation.return_value = False
        self.repo.byLocation.side_effect = IndexError
        self.report(forFile='xyz')
        self.listener.unknownFile.assert_called_with('xyz')

    def test_Can_report_on_stats(self):
        out = self.report(statistics=True)
        self.exporter.export.assert_called_with(self.repo.statistics())
        self.assertEqual(out, self.exporter.export())



